import random


def greet(*args, **kwargs):
    ch = ['hi', 'hello', 'hey', 'howdy', 'hi how are you', 'whats up', 'hey there', 'hello there', 'hey there',
          'hello there', 'hi there']
    return random.choice(ch)



import random
import time


def goodbye(*args, **kwargs):
    ch = ['bye', 'good bye', 'see you later', 'see you', 'good bye']
    night = ['night', 'good night']
    morning = ['morning', 'good morning']
    afternoon = ['afternoon', 'good afternoon']
    evening = ['evening', 'good evening']
    day = ['day', 'good day']
    time_now = time.strftime("%H")
    if "00" <= time_now < "12":
        return random.choice(morning)
    elif "12" <= time_now < "18":
        return random.choice(afternoon)
    elif "18" <= time_now < "24":
        return random.choice(evening)
    elif "24" <= time_now < "06":
        return random.choice(night)
    elif "06" <= time_now < "12":
        return random.choice(day)
    else:
        return random.choice(ch)



import sys

try:
    from services.authenticate import verify_user
    from services.speech_to_text.speech_to_text import speech_to_text_google
    from services.text_to_speech.text_to_speech import text_to_speech
    from services.brain.decision_maker_api import make_decision_nlp, make_decision_ai, make_decision_jarvisai, \
        make_decision_jarvisai_string_matching
    from services.api_management_service.dist.manager import management as management_service
    from CONSTANT import Constant
    from features_default import dict_of_features, what_can_i_do
except:
    from JarvisAI.services.authenticate import verify_user
    from JarvisAI.services.speech_to_text.speech_to_text import speech_to_text_google
    from JarvisAI.services.text_to_speech.text_to_speech import text_to_speech
    from JarvisAI.services.brain.decision_maker_api import make_decision_nlp, make_decision_ai, make_decision_jarvisai, \
        make_decision_jarvisai_string_matching
    from JarvisAI.services.api_management_service.dist.manager import management as management_service
    from JarvisAI.CONSTANT import Constant
    from JarvisAI.features_default import dict_of_features, what_can_i_do

import os
import phonetics
from fuzzywuzzy import fuzz
from playsound import playsound
from download import download

CONSTANT = Constant()


class OutputMethods:
    def __init__(self, backend_tts_api):
        """
        Initialize OutputMethods class
        :param backend_tts_api: str
            backend_tts_api (String) <allowed values: ['pyttsx3', 'gtts']>
        """
        self.backend_tts_api = backend_tts_api

    @staticmethod
    def text_output(*args, **kwargs):
        """
        Print text to console
        """
        text = kwargs.get('text')
        print(text)

    @staticmethod
    def voice_output(*args, **kwargs):
        """
        Play text to speech
        """
        text = kwargs.get('text')
        text_to_speech(text=text, lang='en')


# create a class for inputs methods
class InputsMethods(OutputMethods):
    def __init__(self, backend_tts_api):
        """
        Initialize InputsMethods class
        :param backend_tts_api: str
            backend_tts_api (String) <allowed values: ['pyttsx3', 'gtts']>
        """
        super().__init__(backend_tts_api)

    @staticmethod
    def text_input(*args, **kwargs):
        """
        Get text from user
        """
        input_text = input("Say Something: ")
        return input_text

    @staticmethod
    def voice_input_google_api(*args, **kwargs):
        """
        Get voice input from Google Speech API
        read more about Google Speech API (Pricing and Key) at: https://cloud.google.com/speech-to-text
        """
        google_speech_recognition_input_lang = kwargs.get('google_speech_recognition_input_lang')
        google_speech_recognition_key = kwargs.get('google_speech_recognition_key')
        google_speech_recognition_duration_listening = kwargs.get('google_speech_recognition_duration_listening')
        command, status = speech_to_text_google(input_lang=google_speech_recognition_input_lang,
                                                key=google_speech_recognition_key,
                                                duration=google_speech_recognition_duration_listening)
        if status:
            return command
        else:
            return None

    @staticmethod
    def voice_input_deepspeech_streaming(*args, **kwargs):
        greeting = kwargs.get('greeting', None)
        deepspeech_listen_obj = kwargs.get('deepspeech_listen_obj', None)
        if greeting is not None:
            OutputMethods.voice_output(text=greeting)
        asr_deepspeech_stream = next(deepspeech_listen_obj)
        return asr_deepspeech_stream


class JarvisAI(InputsMethods, OutputMethods):
    def __init__(self, input_method: object, output_method: object, backend_tts_api='pyttsx3', api_key: str = "",
                 detect_wake_word: bool = True, wake_word_detection_method: object = None, bot_name: str = "Jarvis",
                 display_intent: bool = True, google_speech_recognition_input_lang='en',
                 google_speech_recognition_key=None, google_speech_recognition_duration_listening=5):
        """
        :param input_method: (object) method to get input from user <allowed values: [InputsMethods.text_input, InputsMethods.voice_input_google_api, InputsMethods.voice_input_deepspeech_streaming]>
        :param output_method: (object) method to give output to user <allowed values: [OutputMethods.text_output, OutputMethods.voice_output]
        :param backend_tts_api: (str) [Default 'pyttsx3'] backend tts api to use <allowed values: ['pyttsx3', 'gtts']>
        :param api_key: (str) [Default ''] api key to use JarvisAI get it from http://jarvis-ai-api.herokuapp.com
        :param detect_wake_word: (bool) [Default True] detect wake word or not <allowed values: [True, False]>
        :param wake_word_detection_method: (object) [Default None] method to detect wake word <allowed values: [InputsMethods.voice_input_google_api, InputsMethods.voice_input_deepspeech_streaming]
        :param bot_name: (str) [Default 'Jarvis'] name of the bot
        :param display_intent: (bool) [Default True] display intent or not <allowed values: [True, False]>
        :param google_speech_recognition_input_lang: (str) [Default 'en'] language of the input Check supported languages here: https://cloud.google.com/speech-to-text/docs/languages
        :param google_speech_recognition_key: (str) [Default None] api key to use Google Speech API
        :param google_speech_recognition_duration_listening: (int) [Default 5] duration of the listening

        READ MORE: Google Speech API (Pricing and Key) at: https://cloud.google.com/speech-to-text
        """
        self.deepspeech_listen_obj = None
        self.model = None
        self.vad_audio = None
        self.ARGS = None
        self.api_key = api_key
        self.input_method = input_method
        self.output_method = output_method
        self.backend_tts_api = backend_tts_api
        self.detect_wake_word = detect_wake_word
        self.wake_word_detection_method = wake_word_detection_method
        self.bot_name = bot_name
        self.display_intent = display_intent
        self.google_speech_recognition_input_lang = google_speech_recognition_input_lang
        self.google_speech_recognition_key = google_speech_recognition_key
        self.google_speech_recognition_duration_listening = google_speech_recognition_duration_listening
        self.custom_features = {}
        OutputMethods.__init__(self, backend_tts_api=backend_tts_api)

        self.validate_input_output_option()

    def register_feature(self, feature_obj: object, feature_command: str):
        """
        Register a feature to JarvisAI
        :param feature_obj: (object) function that will be called when the feature_command is detected
        :param feature_command: (str) command to activate the function
        """
        if feature_command in list(dict_of_features.keys()):
            raise ValueError(f"Feature command '{feature_command}' already registered. Change the command name.")
        dict_temp = {
            feature_command.lower(): feature_obj
        }
        self.custom_features.update(dict_temp)
        dict_of_features.update(self.custom_features)

    def play_wake_up_sound(self, lock=False):
        if lock:
            if not os.path.exists('ai-off.wav'):
                path = download('https://github.com/Dipeshpal/AdonisAI/raw/main/AdonisAI/utils/ai-off.wav',
                                'ai-off.wav', progressbar=True)
            playsound('ai-off.wav')
        else:
            if not os.path.exists('wake_up.wav'):
                path = download('https://github.com/Dipeshpal/AdonisAI/raw/main/AdonisAI/utils/wake_up.wav',
                                'wake_up.wav', progressbar=True)
            playsound('wake_up.wav')

    def validate_input_output_option(self):
        if self.backend_tts_api not in ['pyttsx3', 'gtts']:
            raise ValueError("Invalid backend_tts_api type. Expected one of: %s" % ['pyttsx3', 'gtts'])
        else:
            if not os.path.exists(CONSTANT.USER_CONFIG_FOLDER):
                os.mkdir(CONSTANT.USER_CONFIG_FOLDER)
            if not os.path.exists(CONSTANT.SPEECH_ENGINE_PATH):
                with open(CONSTANT.SPEECH_ENGINE_PATH, 'w') as f:
                    f.write(self.backend_tts_api)
            else:
                with open(CONSTANT.SPEECH_ENGINE_PATH, 'r') as f:
                    backend_tts_api_tmp = f.read()
                if backend_tts_api_tmp != self.backend_tts_api:
                    with open(CONSTANT.SPEECH_ENGINE_PATH, 'w') as f:
                        f.write(self.backend_tts_api)

        if self.input_method not in [InputsMethods.text_input, InputsMethods.voice_input_google_api,
                                     InputsMethods.voice_input_deepspeech_streaming]:
            raise ValueError("Invalid input_method type. Expected one of: %s" % [InputsMethods.text_input,
                                                                                 InputsMethods.voice_input_google_api,
                                                                                 InputsMethods.voice_input_deepspeech_streaming])
        elif self.input_method in [InputsMethods.voice_input_deepspeech_streaming]:
            try:
                from services.asr_deepspeech_streaming.streaming import listen as deepspeech_listen, ARGS, vad_audio, \
                    model
            except ImportError:
                from JarvisAI.JarvisAI.services.asr_deepspeech_streaming.streaming import listen as deepspeech_listen, \
                    ARGS, \
                    vad_audio, model

            self.ARGS = ARGS
            self.vad_audio = vad_audio
            self.model = model
            self.deepspeech_listen_obj = deepspeech_listen(ARGS, vad_audio, model)

        if self.output_method not in [OutputMethods.text_output, OutputMethods.voice_output]:
            raise ValueError("Invalid output_method type. Expected one of: %s" % [OutputMethods.text_output,
                                                                                  OutputMethods.voice_output])

        if self.api_key is None or self.api_key == "":
            raise ValueError(
                "Invalid api_key. Expected a valid api_key. Get one from https://jarvis-ai-api.herokuapp.com")
        else:
            if not os.path.exists(CONSTANT.USER_CONFIG_FOLDER):
                os.mkdir(CONSTANT.USER_CONFIG_FOLDER)
            if not os.path.exists(CONSTANT.API_KEY_PATH):
                with open(CONSTANT.API_KEY_PATH, 'w') as f:
                    f.write(self.api_key)
            else:
                with open(CONSTANT.API_KEY_PATH, 'r') as f:
                    api_key_tmp = f.read()
                if api_key_tmp != self.api_key:
                    with open(CONSTANT.API_KEY_PATH, 'w') as f:
                        f.write(self.api_key)

        if self.detect_wake_word:
            if self.wake_word_detection_method is None:
                raise ValueError("Invalid wake_word_engine. Expected a valid wake_word_engine. Expected one of: %s" % [
                    InputsMethods.voice_input_google_api,
                    InputsMethods.voice_input_deepspeech_streaming])
            elif self.wake_word_detection_method not in [InputsMethods.voice_input_google_api,
                                                         InputsMethods.voice_input_deepspeech_streaming]:
                raise ValueError(
                    "Invalid wake_word_engine. Expected one of: %s" % [InputsMethods.voice_input_google_api,
                                                                       InputsMethods.voice_input_deepspeech_streaming])
            elif self.wake_word_detection_method in [InputsMethods.voice_input_deepspeech_streaming]:
                if self.deepspeech_listen_obj is None:
                    print("Loading deepspeech model...")
                    try:
                        from services.asr_deepspeech_streaming.streaming import listen as deepspeech_listen, ARGS, \
                            vad_audio, \
                            model
                    except ImportError:
                        from JarvisAI.JarvisAI.services.asr_deepspeech_streaming.streaming import \
                            listen as deepspeech_listen, \
                            ARGS, \
                            vad_audio, model
                    self.ARGS = ARGS
                    self.vad_audio = vad_audio
                    self.model = model
                    self.deepspeech_listen_obj = deepspeech_listen(ARGS, vad_audio, model)

    def manage_tasks(self):
        if not self.detect_wake_word and (self.input_method == InputsMethods.voice_input_google_api or
                                          self.input_method == InputsMethods.voice_input_deepspeech_streaming):
            print("Waiting for your command...")
            # self.play_wake_up_sound()
        inp = self.input_method(
            deepspeech_listen_obj=self.deepspeech_listen_obj,
            google_speech_recognition_input_lang=self.google_speech_recognition_input_lang,
            google_speech_recognition_key=self.google_speech_recognition_key,
            google_speech_recognition_duration_listening=self.google_speech_recognition_duration_listening, )
        if inp is None:
            return
        inp = inp.lower()
        print("You said: " + inp)
        if inp == "stop" or inp == "exit":
            return "stop"

        if inp in list(self.custom_features.keys()):
            accuracy_ = 100
            task = inp
        else:
            task, accuracy_ = make_decision_jarvisai(self.api_key, inp)
        if self.display_intent:
            print("===========>", task.upper(), "with accuracy", accuracy_, "<===========")
        if accuracy_ > 0.5:
            fun = dict_of_features[task]
        else:
            classes = list(dict_of_features.keys())
            task = make_decision_jarvisai_string_matching(classes, inp)
            fun = dict_of_features[task]
            print("===========>", task.upper(), "<===========")
        #     fun = dict_of_features["conversation"]
        # perform action
        if fun is not None:
            call_out = fun(inp)
        else:
            call_out = "Sorry, I don't understand your command."
        if not call_out:
            call_out = "Sorry, I don't understand your command."

        if not self.output_method == OutputMethods.text_output:
            print(call_out)
        self.output_method(text=call_out)
        if inp not in list(self.custom_features.keys()):
            management_service(inp=inp, out=call_out, task=task, secret_key=self.api_key, debug=False)

    @verify_user
    def start(self):
        while True:
            if self.detect_wake_word:
                print("Listening for wake word...")
                wake_word = self.wake_word_detection_method(deepspeech_listen_obj=self.deepspeech_listen_obj)
                if wake_word is None:
                    continue
                if wake_word == "shutdown":
                    print("Shutting down...")
                    sys.exit()
                code1 = phonetics.metaphone(wake_word)
                code2 = phonetics.metaphone(self.bot_name)
                accuracy = fuzz.ratio(code1, code2)
                if len(wake_word) > 0 and accuracy > 50:
                    print("You Said: %s" % self.bot_name)
                    print("Wake word detected...")
                    print("Waiting for your command...")
                    self.play_wake_up_sound()
                    while True:
                        signal = self.manage_tasks()
                        if signal == "stop" or signal == "exit" or signal is None:
                            self.play_wake_up_sound(lock=True)
                            break
                        elif signal == "shutdown":
                            print("Shutting down...")
                            sys.exit()
            else:
                self.manage_tasks()


if __name__ == "__main__":
    # create your own function
    # It must contain parameter 'feature_command' which is the command you want to execute
    # Return is optional
    # If you want to provide return value it should only return text (str)
    # Your return value will be displayed or call out by the choice of OutputMethods of JarvisAI

    def custom_function(
            feature_command="custom command (which is the command you want to execute)"):
        # write your code here to do something with the command
        # perform some tasks
        # return is optional
        return feature_command + ' Executed'


    obj = JarvisAI(input_method=InputsMethods.text_input,
                   output_method=OutputMethods.text_output,
                   backend_tts_api='pyttsx3',
                   api_key="c6fd2013918f9bc9a12c5394a819af49",
                   detect_wake_word=False,
                   wake_word_detection_method=InputsMethods.voice_input_google_api,
                   bot_name="Jarvis",
                   display_intent=True,
                   google_speech_recognition_input_lang='en',
                   google_speech_recognition_key=None,
                   google_speech_recognition_duration_listening=5
                   )

    obj.register_feature(feature_obj=custom_function, feature_command='custom feature')

    obj.start()



import webbrowser


def play_games(inp_command):
    url = 'https://poki.com/'
    try:
        webbrowser.open(url)
        return "Successfully opened Poki.com, Play your games!"
    except Exception as e:
        print(e)
        return "Failed to open Poki.com, please try again!"


if __name__ == '__main__':
    play_games('inp_command')



import json
import requests
import shutup
try:
    from CONSTANT import Constant
except:
    from JarvisAI.CONSTANT import Constant


CONSTANT = Constant()
shutup.please()


def verify_user(method):
    def inner(ref):
        with open(CONSTANT.API_KEY_PATH, 'r') as f:
            api_key = f.read()
        response = requests.get(f'{CONSTANT.API_AUTH_URL}{api_key}')
        if response.status_code == 200:
            status = json.loads(response.content)
            if status:
                print("Authentication Successful")
                return method(ref)
            else:
                print("Authentication Failed. Please check your API key.")
                exit()
        else:
            print(f'Error: {response.status_code}')
            return False

    return inner


if __name__ == '__main__':
    print(verify_user(print))
    print('Done')



try:
    from features.date_time import date_time
    from features.joke import joke
    from features.news import news
    from features.tell_me_about import tell_me_about
    from features.weather import weather
    from features.website_open import website_open
    from features.youtube_play import youtube_play
    from features.whatsapp_message import whatsapp_message
    from features.send_email import send_email
    from features.chatbot import chatbot
    from features.chatbot import greet
    from features.chatbot import goodbye
    from features.screenshot import screenshot
    from features.click_photo import click_photo
    # from features.internet_speed_test import internet_speed_test
    from features.youtube_video_downloader import youtube_video_downloader
    from features.covid_cases import covid_cases
    from features.games import games
    from features.places_near_me import places_near_me
    from features.iambored import iambored
    from features.volume_control import volume_controller
except ImportError as e:
    # print("ImportError: {}".format(e))
    from .features.date_time import date_time
    from .features.joke import joke
    from .features.news import news
    from .features.tell_me_about import tell_me_about
    from .features.weather import weather
    from .features.website_open import website_open
    from .features.youtube_play import youtube_play
    from .features.whatsapp_message import whatsapp_message
    from .features.send_email import send_email
    from .features.chatbot import chatbot
    from .features.chatbot import greet
    from .features.chatbot import goodbye
    from .features.screenshot import screenshot
    from .features.click_photo import click_photo
    # from .features.internet_speed_test import internet_speed_test
    from .features.youtube_video_downloader import youtube_video_downloader
    from .features.covid_cases import covid_cases
    from .features.games import games
    from .features.places_near_me import places_near_me
    from .features.iambored import iambored
    from .features.volume_control import volume_controller


dict_of_features = {
    'asking date': date_time.date,
    'asking time': date_time.time,
    'tell me joke': joke.tell_me_joke,
    'tell me news': news.news,
    'asking weather': weather.get_weather,
    'tell me about': tell_me_about.tell_me_about,
    'open website': website_open.website_opener,
    'play on youtube': youtube_play.yt_play,
    'send whatsapp message': whatsapp_message.send_whatsapp_message,
    'send email': send_email.send_email,
    'greet': greet.greet,
    'goodbye': goodbye.goodbye,
    # 'conversation': chatbot.chatbot_general_purpose,
    'take screenshot': screenshot.take_screenshot,
    'click photo': click_photo.click_pic,
    # 'check internet speed': internet_speed_test.speed_test,
    'download youtube video': youtube_video_downloader.download_yt_video,
    'covid cases': covid_cases.check_command_is_for_covid_cases,  #
    'play games': games.play_games,
    'places near me': places_near_me.get_places_near_me,
    'i am bored': iambored.get_me_suggestion,
    'volume control': volume_controller.start_volume_control,
}

what_can_i_do = {
    'you can ask me date ': 'Say- "what is the date today"',
    'you can ask me time ': 'Say- "what is the time now"',
    'you can ask me joke ': 'Say- "tell me a joke"',
    'you can ask me news ': 'Say- "tell me news"',
    'you can ask me weather ': 'Say- "what is the weather", "tell me weather", "tell me about weather",'
                               ' "what is the weather in <city>"',
    'you can ask me about ': 'Say- "tell me about <topic>"',
    'you can open website ': 'Say- "open websiteopen website <website name>", "open website <website name><.extension>"'
                             '"open website techport.in"',
    'you can play on youtube ': 'Say- "play on youtube <video name>", "play <video name> on youtube"',
    'you can send whatsapp message ': 'Say- "send whatsapp message',
    'you can send email ': 'Say- "send email',
    'greet': 'Say- "greet", "hello", "hey", "hi", "good morning", "good afternoon", "good evening"',
    'goodbye': 'Say- "goodbye", "bye", "see you later"',
    # 'conversation': 'Say- "conversation", "chat", "talk", "talk with chatbot"',
    'you can take a screenshot of current screen': 'Say- "take screenshot"',
    'you can click a photo': 'Say- "click photo"',
    # 'you can check internet speed': 'Say- "check internet speed"',
    'you can download youtube video': 'Say- "download youtube video"',
    'you can check covid cases': 'Say- "covid cases in <country>", "covid cases <country>"',
    'you can ask to play games': 'Say- "play games"',
    'you can ask places near me': 'Say- "cafe near me"',
    'you can ask to suggest something': 'Say- "i am bored"',
    'you can control volume': 'Say- "open volume control"'
}

if __name__ == '__main__':
    print(', '.join(list(dict_of_features.keys())))



from pprint import pprint

import requests
import numpy as np
import os

try:
    from CONSTANT import Constant
except:
    from JarvisAI.CONSTANT import Constant

CONSTANT = Constant()


def chatbot_only_chat(message):
    if not os.path.exists(CONSTANT.API_KEY_PATH):
        return f"""Please create a {CONSTANT.API_KEY_PATH} file with your secret key in it.
        It's used for security purpose.
        Get your free key from "{CONSTANT.BASE_URL}"
        """
    else:
        with open(CONSTANT.API_KEY_PATH, "r") as f:
            secret_key = f.read()

    if os.path.exists('step.npy'):
        step = str(np.load('step.npy'))
    else:
        step = '0'
    url = f"{CONSTANT.BASE_URL}chatbot_api_only_chat?secret_key={secret_key}&message={message}&step={step}"
    response = requests.get(url)
    res = response.json()
    status = res['status']

    if status:
        step = int(res['response']['data'][0]['step'])
        np.save('step.npy', step)
        return res['response']['data'][0]['res']
    else:
        return res['response'] + '\n' + f"Get a new secret key from {CONSTANT.BASE_URL}"


def chatbot_general_purpose(message: str, google_result: bool = False, youtube_result: bool = False, *args, **kwargs):
    if not os.path.exists(CONSTANT.API_KEY_PATH):
        return f"""Please create a {CONSTANT.API_KEY_PATH} file with your secret key in it.
        It's used for security purpose.
        Get your free key from "{CONSTANT.BASE_URL}"
        """
    else:
        with open(CONSTANT.API_KEY_PATH, "r") as f:
            secret_key = f.read()

    if os.path.exists('step.npy'):
        step = str(np.load('step.npy'))
    else:
        step = '0'

    url = f"{CONSTANT.BASE_URL}chatbot_api_general_purpose?secret_key={secret_key}&query={message}&google_result={google_result}&youtube_result={youtube_result}"
    response = requests.get(url)
    try:
        res = response.json()
        if res['status']:
            step = int(res['response']['response']['data'][0][0]['weights_of_all_models']['conversation'][-1]['step'])
            np.save('step.npy', step)
            conversation = res['response']['response']['data'][0][0]['weights_of_all_models']['conversation'][0]

            suggested_answer = res['response']['response']['data'][0][0]['suggested_answer']['answer']

            return suggested_answer
        else:
            return "Something went wrong with chatbot_api_general_purpose server"
    except Exception as e:
        return f"Something went wrong with chatbot_api_general_purpose server {e}"


if __name__ == "__main__":
    # for i in range(7):
    pprint(chatbot_general_purpose("weather in Madhya Pradesh"))



import logging
from datetime import datetime
import collections, queue, os, os.path
import deepspeech
import numpy as np
import pyaudio
import wave
import webrtcvad
from halo import Halo
from scipy import signal
from download import download

logging.basicConfig(level=20)


class Audio(object):
    """Streams raw audio from microphone. Data is received in a separate thread, and stored in a buffer, to be read from."""

    FORMAT = pyaudio.paInt16
    # Network/VAD rate-space
    RATE_PROCESS = 16000
    CHANNELS = 1
    BLOCKS_PER_SECOND = 50

    def __init__(self, callback=None, device=None, input_rate=RATE_PROCESS, file=None):
        def proxy_callback(in_data, frame_count, time_info, status):
            # pylint: disable=unused-argument
            if self.chunk is not None:
                in_data = self.wf.readframes(self.chunk)
            callback(in_data)
            return (None, pyaudio.paContinue)

        if callback is None: callback = lambda in_data: self.buffer_queue.put(in_data)
        self.buffer_queue = queue.Queue()
        self.device = device
        self.input_rate = input_rate
        self.sample_rate = self.RATE_PROCESS
        self.block_size = int(self.RATE_PROCESS / float(self.BLOCKS_PER_SECOND))
        self.block_size_input = int(self.input_rate / float(self.BLOCKS_PER_SECOND))
        self.pa = pyaudio.PyAudio()

        kwargs = {
            'format': self.FORMAT,
            'channels': self.CHANNELS,
            'rate': self.input_rate,
            'input': True,
            'frames_per_buffer': self.block_size_input,
            'stream_callback': proxy_callback,
        }

        self.chunk = None
        # if not default device
        if self.device:
            kwargs['input_device_index'] = self.device
        elif file is not None:
            self.chunk = 320
            self.wf = wave.open(file, 'rb')

        self.stream = self.pa.open(**kwargs)
        self.stream.start_stream()

    def resample(self, data, input_rate):
        """
        Microphone may not support our native processing sampling rate, so
        resample from input_rate to RATE_PROCESS here for webrtcvad and
        deepspeech

        Args:
            data (binary): Input audio stream
            input_rate (int): Input audio rate to resample from
        """
        data16 = np.fromstring(string=data, dtype=np.int16)
        resample_size = int(len(data16) / self.input_rate * self.RATE_PROCESS)
        resample = signal.resample(data16, resample_size)
        resample16 = np.array(resample, dtype=np.int16)
        return resample16.tostring()

    def read_resampled(self):
        """Return a block of audio data resampled to 16000hz, blocking if necessary."""
        return self.resample(data=self.buffer_queue.get(),
                             input_rate=self.input_rate)

    def read(self):
        """Return a block of audio data, blocking if necessary."""
        return self.buffer_queue.get()

    def destroy(self):
        self.stream.stop_stream()
        self.stream.close()
        self.pa.terminate()

    frame_duration_ms = property(lambda self: 1000 * self.block_size // self.sample_rate)

    def write_wav(self, filename, data):
        logging.info("write wav %s", filename)
        wf = wave.open(filename, 'wb')
        wf.setnchannels(self.CHANNELS)
        # wf.setsampwidth(self.pa.get_sample_size(FORMAT))
        assert self.FORMAT == pyaudio.paInt16
        wf.setsampwidth(2)
        wf.setframerate(self.sample_rate)
        wf.writeframes(data)
        wf.close()


class VADAudio(Audio):
    """Filter & segment audio with voice activity detection."""

    def __init__(self, aggressiveness=3, device=None, input_rate=None, file=None):
        super().__init__(device=device, input_rate=input_rate, file=file)
        self.vad = webrtcvad.Vad(aggressiveness)

    def frame_generator(self):
        """Generator that yields all audio frames from microphone."""
        if self.input_rate == self.RATE_PROCESS:
            while True:
                yield self.read()
        else:
            while True:
                yield self.read_resampled()

    def vad_collector(self, padding_ms=300, ratio=0.75, frames=None):
        """Generator that yields series of consecutive audio frames comprising each utterence, separated by yielding a single None.
            Determines voice activity by ratio of frames in padding_ms. Uses a buffer to include padding_ms prior to being triggered.
            Example: (frame, ..., frame, None, frame, ..., frame, None, ...)
                      |---utterence---|        |---utterence---|
        """
        if frames is None: frames = self.frame_generator()
        num_padding_frames = padding_ms // self.frame_duration_ms
        ring_buffer = collections.deque(maxlen=num_padding_frames)
        triggered = False

        for frame in frames:
            if len(frame) < 640:
                return

            is_speech = self.vad.is_speech(frame, self.sample_rate)

            if not triggered:
                ring_buffer.append((frame, is_speech))
                num_voiced = len([f for f, speech in ring_buffer if speech])
                if num_voiced > ratio * ring_buffer.maxlen:
                    triggered = True
                    for f, s in ring_buffer:
                        yield f
                    ring_buffer.clear()

            else:
                yield frame
                ring_buffer.append((frame, is_speech))
                num_unvoiced = len([f for f, speech in ring_buffer if not speech])
                if num_unvoiced > ratio * ring_buffer.maxlen:
                    triggered = False
                    yield None
                    ring_buffer.clear()


def listen(ARGS, vad_audio, model):
    print("Listening (ctrl-C to exit) or Close/Terminate the window...")
    frames = vad_audio.vad_collector()

    # Stream from microphone to DeepSpeech using VAD
    spinner = None
    if not ARGS.nospinner:
        spinner = Halo(spinner='line')
    stream_context = model.createStream()
    wav_data = bytearray()
    for frame in frames:
        if frame is not None:
            if spinner: spinner.start()
            logging.debug("streaming frame")
            stream_context.feedAudioContent(np.frombuffer(frame, np.int16))
            if ARGS.savewav: wav_data.extend(frame)
        else:
            if spinner: spinner.stop()
            logging.debug("end utterence")
            if ARGS.savewav:
                vad_audio.write_wav(
                    os.path.join(ARGS.savewav, datetime.now().strftime("savewav_%Y-%m-%d_%H-%M-%S_%f.wav")), wav_data)
                wav_data = bytearray()
            text = stream_context.finishStream()
            # print("Recognized-: %s" % text)
            stream_context = model.createStream()
            yield text


def main(ARGS):
    # Load DeepSpeech model
    if os.path.isdir(ARGS.model):
        model_dir = ARGS.model
        ARGS.model = os.path.join(model_dir, 'output_graph.pb')
        ARGS.scorer = os.path.join(model_dir, ARGS.scorer)

    print('Initializing model...')
    logging.info("ARGS.model: %s", ARGS.model)
    model = deepspeech.Model(ARGS.model)
    if ARGS.scorer:
        logging.info("ARGS.scorer: %s", ARGS.scorer)
        model.enableExternalScorer(ARGS.scorer)

    # Start audio with VAD
    vad_audio = VADAudio(aggressiveness=ARGS.vad_aggressiveness,
                         device=ARGS.device,
                         input_rate=ARGS.rate,
                         file=ARGS.file)
    return ARGS, vad_audio, model


DEFAULT_SAMPLE_RATE = 16000
vad_aggressiveness = 0
nospinner = False
savewav = None
file = None
model = 'deepspeech-0.9.3-models.pbmm'
scorer = None
device = 0
if not os.path.exists(model):
    url = 'https://github.com/mozilla/DeepSpeech/releases/download/v0.9.3/deepspeech-0.9.3-models.pbmm'
    print('Model does not exist, downloading from {}'.format(url))
    download(url, model, progressbar=True)

import argparse

ARGS, vad_audio, model = main(
    argparse.Namespace(model=model, scorer=scorer, device=device, file=file, savewav=savewav, nospinner=nospinner,
                       vad_aggressiveness=vad_aggressiveness, rate=DEFAULT_SAMPLE_RATE))


def asr_streaming():
    return listen(ARGS, vad_audio, model)
    # a = listen(ARGS, vad_audio, model)


if __name__ == '__main__':
    a = asr_streaming()
    while True:
        if next(a) is not None:
            print(next(a))



from urllib.request import urlopen
from bs4 import BeautifulSoup as soup
import webbrowser


def news(*args, **kwargs):
    """
    This method will tells top 15 current NEWS
    :return: list / bool
    """
    try:
        news_url = "https://news.google.com/news/rss"
        Client = urlopen(news_url)
        xml_page = Client.read()
        Client.close()
        soup_page = soup(xml_page, "xml")
        news_list = soup_page.findAll("item")
        li = []
        for news in news_list[:5]:
            li.append(str(news.title.text.encode('utf-8'))[1:])
        return "\n".join(li).strip()
    except Exception as e:
        print(e)
        return False


def show_me_some_tech_news():
    try:
        url = "https://thetechport.in/"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


def show_me_some_tech_videos():
    try:
        url = "https://www.youtube.com/c/TechPortOfficial"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


if __name__ == "__main__":
    print(news())
    print(show_me_some_tech_news())


import os
from gtts import gTTS
import pyttsx3
from playsound import playsound
from lazyme.string import color_print as cprint

USER_CONFIG_FOLDER = './user_configs/'
SPEECH_ENGINE_PATH = f'{USER_CONFIG_FOLDER}speech_engine.txt'


def text_to_speech(text, lang='en'):
    """
    Convert any text to speech
    You can use GTTS or PYTTSX3 as backend for Text to Speech.
    PYTTSX3 may support different voices (male/female) depends upon your system.
    You can set backend of tts while creating object of JarvisAI class. Default is PYTTSX3.
    :param text: str
        text (String)
    :param lang: str
        default 'en'
    :return: Bool
        True / False (Play sound if True otherwise write exception to log and return False)
    """
    with open(SPEECH_ENGINE_PATH, 'r') as f:
        backend_tts_api = f.read()
    if backend_tts_api == 'gtts':
        # for gtts Backend
        try:
            myobj = gTTS(text=text, lang=lang, slow=False)
            myobj.save("tmp.mp3")
            playsound("tmp.mp3")
            os.remove("tmp.mp3")
            return True
        except Exception as e:
            print(e)
            print("or You may reached free limit of 'gtts' API. Use 'pyttsx3' as backend for unlimited use.")
            return False
    else:
        # for pyttsx3 Backend
        engine = pyttsx3.init()
        voices = engine.getProperty('voices')

        try:
            if not os.path.exists("configs"):
                os.mkdir("configs")

            voice_file_name = "configs/Edith-Voice.txt"
            if not os.path.exists(voice_file_name):
                cprint("You can try different voices. This is one time setup. You can reset your voice by deleting"
                       "'configs/Edith-Voice.txt' file in your working directory.",
                       color='blue')
                cprint("Your System Support Following Voices- ",
                       color='blue')
                voices_dict = {}
                for index, voice in enumerate(voices):
                    print(f"{index}: ", voice.id)
                    voices_dict[str(index)] = voice.id
                option = input(f"Choose any- {list(voices_dict.keys())}: ")
                with open(voice_file_name, 'w') as f:
                    f.write(voices_dict.get(option, voices[0].id))
                with open(voice_file_name, 'r') as f:
                    voice_property = f.read()
            else:
                with open(voice_file_name, 'r') as f:
                    voice_property = f.read()
        except Exception as e:
            print(e)
            print("Error occurred while creating config file for voices in pyttsx3 in 'text2speech'.",
                  "Contact maintainer/developer of JarvisAI")
        try:
            engine.setProperty('voice', voice_property)
            engine.say(text)
            engine.runAndWait()
            return True
        except Exception as e:
            print(e)
            print("Error occurred while using pyttsx3 in 'text2speech'.",
                  "or Your system may not support pyttsx3 backend. Use 'gtts' as backend.",
                  "Contact maintainer/developer of JarvisAI.")
            return False



import setuptools
from setuptools import find_namespace_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="JarvisAI",
    version="4.1",
    author="Dipesh",
    author_email="dipeshpal17@gmail.com",
    description="JarvisAI is python library to build your own AI virtual assistant with natural language processing.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dipeshpal/Jarvis_AI",
    include_package_data=True,
    packages=find_namespace_packages(include=['JarvisAI.*', 'JarvisAI']),
    install_requires=['numpy', 'gtts', 'playsound', 'pyscreenshot', "opencv-python",
                      'SpeechRecognition', 'pipwin', 'lxml', 'pyjokes',
                      'beautifulsoup4', 'wikipedia', 'scipy', 'download',
                      "torch", 'lazyme', "requests", "pyttsx3", "googlesearch-python",
                      "spacy", 'textdistance', 'pywhatkit', "googlesearch-python",
                      "youtube-search-python", "shutup", 'Flask', 'speedtest-cli',
                      'pytube', 'pycountry', 'phonetics', 'fuzzywuzzy', 'googletrans', 'wave',
                      'deepspeech', 'halo', 'playsound==1.2.2', 'pyaudio', 'mediapipe==0.8.11',
                      'pycaw'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    project_urls={
        'Documentation': 'https://github.com/Dipeshpal/Jarvis_AI',
        'Donate': 'https://www.buymeacoffee.com/dipeshpal',
        'Say Thanks!': 'https://youtube.com/techportofficial',
        'Source': 'https://github.com/Dipeshpal/Jarvis_AI',
    },
)
